/**
 * Routines and data for LED handling
 */

#include <stdint.h>
#include <avr/interrupt.h>
#include <avr/io.h>

// defines port bits for a certain LED segment
#define LED_A PD6
#define LED_B PD3
#define LED_C PD1
#define LED_D PD0
#define LED_E PD4
#define LED_F PD5
#define LED_G PD2

// bytes containing 7-segment output data
volatile uint8_t out_bytes[4];

// take over RPM value
void LEDsetRPM(uint16_t rpm)
{
  // limit value to maximum
  if(rpm > 9999)
    {
      rpm = 9999;
    }
  // set digit bytes to correct value
  for(int i=0; i<4; i++)
    {
      // begin with lowest digit
      switch(rpm % 10)
	{
	case 0:
	  // only display zero if it is not a leading zero
	  if(i!=0 && rpm < 10)
	    {
	      out_bytes[i] = 0;
	    }
	  else
	    {
	      out_bytes[i] = (1<<LED_A) | (1<<LED_B) | (1<<LED_C) |
		(1<<LED_D) | (1<<LED_E) | (1<<LED_F);
	    }
	  break;
	case 1:
	  out_bytes[i] = (1<<LED_B) | (1<<LED_C);
	  break;
	case 2:
	  out_bytes[i] = (1<<LED_A) | (1<<LED_B) | (1<<LED_G) |
	    (1<<LED_E) | (1<<LED_D);
	  break;
	case 3:
	  out_bytes[i] = (1<<LED_A) | (1<<LED_B) | (1<<LED_G) |
	    (1<<LED_C) | (1<<LED_D);
	  break;
	case 4:
	  out_bytes[i] = (1<<LED_F) | (1<<LED_G) |
	    (1<<LED_B) | (1<<LED_C);
	  break;       
	case 5:
	  out_bytes[i] = (1<<LED_A) | (1<<LED_F) | (1<<LED_G) |
	    (1<<LED_C) | (1<<LED_D);
	  break;
	case 6:
	  out_bytes[i] = (1<<LED_A) | (1<<LED_F) | (1<<LED_E) |
	    (1<<LED_D) | (1<<LED_C) | (1<<LED_G);
	  break;
	case 7:
	  out_bytes[i] = (1<<LED_A) | (1<<LED_B) | (1<<LED_C);
	  break;
	case 8:
	  out_bytes[i] = (1<<LED_A) | (1<<LED_B) | (1<<LED_C) |
	    (1<<LED_D) | (1<<LED_E) | (1<<LED_F) | (1<<LED_G);
	  break;
	case 9:
	  out_bytes[i] = (1<<LED_G) | (1<<LED_F) | (1<<LED_A) |
	    (1<<LED_B) | (1<<LED_C) | (1<<LED_D);
	  break;
	}
      // for next output digit, take next input digit
      rpm /= 10;
    }
}

// initialize LED output
void LEDinit(void)
{
  // make all LED ports outputs
  DDRD |= (1<<PD0) | (1<<PD1) | (1<<PD2) | (1<<PD3)
    | (1<<PD4) | (1<<PD5) | (1<<PD6);
  DDRC |= (1<<PC0) | (1<<PC1) | (1<<PC2) | (1<<PC3);

  // enable timer0 overflow IRQ, prescaler = 64 -> IRQ freq = 488Hz
  TCCR0 = (1<<CS01) | (1<<CS00);
  TIMSK |= (1<<TOIE0);

  for(uint8_t i=0; i<4; i++)
    {
      out_bytes[i] = 0;
    }
}

// timer 0 overflow IRQ handler - switch to next digit
ISR(TIMER0_OVF_vect)
{
  // currently active digit
  static uint8_t digit;

  // switch off output
  PORTD = 0;
  // switch to next digit in output array
  digit++;
  // ...and in port
  PORTC <<= 1;
  // check if last digit reached
  if(digit >= 4)
    {
      digit = 0;
      PORTC = (1<<PC0);
    }
  // output digit data
  PORTD = out_bytes[digit];
}
