/**
 * Manage input capture IRQs and calculate RPM value from timers
 */

#include <stdint.h>
#include <avr/interrupt.h>
#include <avr/io.h>
#include <util/atomic.h>

// the timestamp of the previous IGN-pulse
volatile uint16_t prev;
// the timestamp of the current IGN-pulse
volatile uint16_t current;
// overflows between prev and current
volatile uint8_t overflows;
// overflows since current
volatile uint8_t overflow_counter;

uint16_t RPMgetRPM(void)
{
  uint16_t local_prev, local_current;
  uint8_t local_overflows;

  ATOMIC_BLOCK(ATOMIC_FORCEON)
    {
      local_prev = prev;
      local_current = current;
      local_overflows = overflows;
    }
  // check for timeout
  if(local_overflows > 1 || (local_overflows == 1 && local_current >= local_prev) )
    {
      return 0;
    }
  else
    {
      // time between two IGN pulses
      uint16_t delta = local_current - local_prev;
      // 60 seconds, 1000000 timer ticks per second
      return UINT32_C(60) * UINT32_C(1000000) / delta
	// two IGN pulses per rotation
	/ 2; 
    }
}

void RPMinit(void)
{
  // enable timer 1, prescaler 8 -> 1us timestep
  // with overflow IRQ, input capture IRQ on falling edge, noise canceler
  TCCR1A = 0;
  TCCR1B = (1<<ICNC1) | (1<<CS11);
  TIMSK |= (1<<TOIE1) | (1<<TICIE1);

  // set overflow variables to some high value so initial rpm is 0
  overflows = overflow_counter = 2;
}

// timer overflow IRQ
ISR(TIMER1_OVF_vect)
{
  // increase overflow counter, if not already ready
  if(overflow_counter < 2)
    {
      overflow_counter++;
    }
  else
    {
      // if two many overflows accumulated, show in value
      overflows = overflow_counter;
    }
}

// timer output compare IRQ - delay next IGN-pulse
ISR(TIMER1_COMP_vect)
{
  // disable output compare IRQ
  TIMSK &= ~(1<<OCIE1);
  // clear input capture IRQ flag
  TIFR |= (1<<ICF1);
  // enable input capture IRQ
  TIMSK |= (1<<TICIE1);
}

// timer input capture IRQ - IGN pulse detected
ISR(TIMER1_CAPT_vect)
{
  // disable input capture IRQ
  TIMSK &= ~(1<<TICIE1);
  // enable output compare IRQ to reenable input capture IRQ in 2ms
  OCR1 = ICR1 + 2000;
  TIFR |= (1<<OCF1);
  TIMSK |= (1<<OCIE1);

  // finally read in capture value
  prev = current;
  current = ICR1;

  // and clear overflow_counter, transfer its value
  overflows = overflow_counter;
  overflow_counter = 0;
}
